@@ -35,6 +35,7 @@
 #include "xpc_make_class.h"
 #include "XPCWrapper.h"
 #include "Crypto.h"
+#include "mozilla/Result.h"
 #include "mozilla/dom/AbortControllerBinding.h"
 #include "mozilla/dom/AutoEntryScript.h"
 #include "mozilla/dom/BindingCallContext.h"
@@ -59,6 +60,8 @@
 #include "mozilla/dom/InspectorUtilsBinding.h"
 #include "mozilla/dom/MessageChannelBinding.h"
 #include "mozilla/dom/MessagePortBinding.h"
+#include "mozilla/dom/MIDIInputMapBinding.h"
+#include "mozilla/dom/MIDIOutputMapBinding.h"
 #include "mozilla/dom/ModuleLoader.h"
 #include "mozilla/dom/NodeBinding.h"
 #include "mozilla/dom/NodeFilterBinding.h"
@@ -78,9 +81,9 @@
 #include "mozilla/dom/ScriptSettings.h"
 #include "mozilla/dom/SelectionBinding.h"
 #include "mozilla/dom/StorageManager.h"
+#include "mozilla/dom/StorageManagerBinding.h"
 #include "mozilla/dom/TextDecoderBinding.h"
 #include "mozilla/dom/TextEncoderBinding.h"
-#include "mozilla/dom/UnionConversions.h"
 #include "mozilla/dom/URLBinding.h"
 #include "mozilla/dom/URLSearchParamsBinding.h"
 #include "mozilla/dom/XMLHttpRequest.h"
@@ -89,6 +92,8 @@
 #include "mozilla/dom/XMLSerializerBinding.h"
 #include "mozilla/dom/FormDataBinding.h"
 #include "mozilla/dom/nsCSPContext.h"
+#include "mozilla/ipc/BackgroundUtils.h"
+#include "mozilla/ipc/PBackgroundSharedTypes.h"
 #include "mozilla/BasePrincipal.h"
 #include "mozilla/DeferredFinalize.h"
 #include "mozilla/ExtensionPolicyService.h"
@@ -98,6 +103,7 @@
 #include "mozilla/StaticPrefs_extensions.h"
 
 using namespace mozilla;
+using namespace mozilla::dom;
 using namespace JS;
 using namespace JS::loader;
 using namespace xpc;
@@ -105,7 +111,7 @@ using namespace xpc;
 using mozilla::dom::DestroyProtoAndIfaceCache;
 using mozilla::dom::IndexedDatabaseManager;
 
-NS_IMPL_CYCLE_COLLECTION_CLASS(SandboxPrivate)
+NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE_CLASS(SandboxPrivate)
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(SandboxPrivate)
   NS_IMPL_CYCLE_COLLECTION_UNLINK_PRESERVED_WRAPPER
@@ -120,8 +126,6 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(SandboxPrivate)
   tmp->TraverseObjectsInGlobal(cb);
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
-NS_IMPL_CYCLE_COLLECTION_TRACE_WRAPPERCACHE(SandboxPrivate)
-
 NS_IMPL_CYCLE_COLLECTING_ADDREF(SandboxPrivate)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(SandboxPrivate)
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(SandboxPrivate)
@@ -270,7 +274,7 @@ static bool SandboxImport(JSContext* cx, unsigned argc, Value* vp) {
   return true;
 }
 
-static bool SandboxCreateCrypto(JSContext* cx, JS::HandleObject obj) {
+bool xpc::SandboxCreateCrypto(JSContext* cx, JS::Handle<JSObject*> obj) {
   MOZ_ASSERT(JS_IsGlobalObject(obj));
 
   nsIGlobalObject* native = xpc::NativeGlobal(obj);
@@ -297,23 +301,6 @@ static bool SandboxCreateRTCIdentityProvider(JSContext* cx,
 }
 #endif
 
-static bool SetFetchRequestFromValue(JSContext* cx, RequestOrUSVString& request,
-                                     const MutableHandleValue& requestOrUrl) {
-  RequestOrUSVStringArgument requestHolder(request);
-  bool noMatch = true;
-  if (requestOrUrl.isObject() &&
-      !requestHolder.TrySetToRequest(cx, requestOrUrl, noMatch, false)) {
-    return false;
-  }
-  if (noMatch && !requestHolder.TrySetToUSVString(cx, requestOrUrl, noMatch)) {
-    return false;
-  }
-  if (noMatch) {
-    return false;
-  }
-  return true;
-}
-
 static bool SandboxFetch(JSContext* cx, JS::HandleObject scope,
                          const CallArgs& args) {
   if (args.length() < 1) {
@@ -321,14 +308,13 @@ static bool SandboxFetch(JSContext* cx, JS::HandleObject scope,
     return false;
   }
 
+  BindingCallContext callCx(cx, "fetch");
   RequestOrUSVString request;
-  if (!SetFetchRequestFromValue(cx, request, args[0])) {
-    JS_ReportErrorASCII(cx, "fetch requires a string or Request in argument 1");
+  if (!request.Init(callCx, args[0], "Argument 1")) {
     return false;
   }
   RootedDictionary<dom::RequestInit> options(cx);
-  BindingCallContext callCx(cx, "fetch");
-  if (!options.Init(cx, args.hasDefined(1) ? args[1] : JS::NullHandleValue,
+  if (!options.Init(callCx, args.hasDefined(1) ? args[1] : JS::NullHandleValue,
                     "Argument 2", false)) {
     return false;
   }
@@ -359,13 +345,13 @@ static bool SandboxFetchPromise(JSContext* cx, unsigned argc, Value* vp) {
   return ConvertExceptionToPromise(cx, args.rval());
 }
 
-static bool SandboxCreateFetch(JSContext* cx, HandleObject obj) {
+bool xpc::SandboxCreateFetch(JSContext* cx, JS::Handle<JSObject*> obj) {
   MOZ_ASSERT(JS_IsGlobalObject(obj));
 
   return JS_DefineFunction(cx, obj, "fetch", SandboxFetchPromise, 2, 0) &&
-         dom::Request_Binding::GetConstructorObject(cx) &&
-         dom::Response_Binding::GetConstructorObject(cx) &&
-         dom::Headers_Binding::GetConstructorObject(cx);
+         Request_Binding::CreateAndDefineOnGlobal(cx) &&
+         Response_Binding::CreateAndDefineOnGlobal(cx) &&
+         Headers_Binding::CreateAndDefineOnGlobal(cx);
 }
 
 static bool SandboxCreateStorage(JSContext* cx, JS::HandleObject obj) {
@@ -374,6 +360,10 @@ static bool SandboxCreateStorage(JSContext* cx, JS::HandleObject obj) {
   nsIGlobalObject* native = xpc::NativeGlobal(obj);
   MOZ_ASSERT(native);
 
+  if (!StorageManager_Binding::CreateAndDefineOnGlobal(cx)) {
+    return false;
+  }
+
   dom::StorageManager* storageManager = new dom::StorageManager(native);
   JS::RootedObject wrapped(cx, storageManager->WrapObject(cx, nullptr));
   return JS_DefineProperty(cx, obj, "storage", wrapped, JSPROP_ENUMERATE);
@@ -412,7 +402,7 @@ static bool SandboxStructuredClone(JSContext* cx, unsigned argc, Value* vp) {
   return true;
 }
 
-static bool SandboxCreateStructuredClone(JSContext* cx, HandleObject obj) {
+bool xpc::SandboxCreateStructuredClone(JSContext* cx, HandleObject obj) {
   MOZ_ASSERT(JS_IsGlobalObject(obj));
 
   return JS_DefineFunction(cx, obj, "structuredClone", SandboxStructuredClone,
@@ -942,6 +932,10 @@ bool xpc::GlobalProperties::Parse(JSContext* cx, JS::HandleObject obj) {
       InspectorUtils = true;
     } else if (JS_LinearStringEqualsLiteral(nameStr, "MessageChannel")) {
       MessageChannel = true;
+    } else if (JS_LinearStringEqualsLiteral(nameStr, "MIDIInputMap")) {
+      MIDIInputMap = true;
+    } else if (JS_LinearStringEqualsLiteral(nameStr, "MIDIOutputMap")) {
+      MIDIOutputMap = true;
     } else if (JS_LinearStringEqualsLiteral(nameStr, "Node")) {
       Node = true;
     } else if (JS_LinearStringEqualsLiteral(nameStr, "NodeFilter")) {
@@ -1017,133 +1011,60 @@ bool xpc::GlobalProperties::Define(JSContext* cx, JS::HandleObject obj) {
   // This function holds common properties not exposed automatically but able
   // to be requested either in |Cu.importGlobalProperties| or
   // |wantGlobalProperties| of a sandbox.
-  if (AbortController &&
-      !dom::AbortController_Binding::GetConstructorObject(cx)) {
-    return false;
+
+#define DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(_iface)                     \
+  if ((_iface) && !dom::_iface##_Binding::CreateAndDefineOnGlobal(cx)) { \
+    return false;                                                        \
   }
 
-  if (Blob && !dom::Blob_Binding::GetConstructorObject(cx)) return false;
-
-  if (ChromeUtils && !dom::ChromeUtils_Binding::GetConstructorObject(cx)) {
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(AbortController)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(ChromeUtils)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(Blob)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(CSS)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(CSSRule)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(Directory)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(Document)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(DOMException)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(DOMParser)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(DOMTokenList)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(Element)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(Event)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(File)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(FileReader)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(FormData)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(Headers)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(IOUtils)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(InspectorUtils)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(MessageChannel)
+  if (MessageChannel && !MessagePort_Binding::CreateAndDefineOnGlobal(cx)) {
     return false;
   }
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(MIDIInputMap)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(MIDIOutputMap)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(Node)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(NodeFilter)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(PathUtils)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(Performance)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(PromiseDebugging)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(Range)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(ReadableStream)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(Selection)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(TextDecoder)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(TextEncoder)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(URL)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(URLSearchParams)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(WebSocket)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(Window)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(XMLHttpRequest)
+  DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE(XMLSerializer)
 
-  if (CSS && !dom::CSS_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (CSSRule && !dom::CSSRule_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (Directory && !dom::Directory_Binding::GetConstructorObject(cx))
-    return false;
-
-  if (Document && !dom::Document_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (DOMException && !dom::DOMException_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (DOMParser && !dom::DOMParser_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (DOMTokenList && !dom::DOMTokenList_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (Element && !dom::Element_Binding::GetConstructorObject(cx)) return false;
-
-  if (Event && !dom::Event_Binding::GetConstructorObject(cx)) return false;
-
-  if (File && !dom::File_Binding::GetConstructorObject(cx)) return false;
-
-  if (FileReader && !dom::FileReader_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (FormData && !dom::FormData_Binding::GetConstructorObject(cx))
-    return false;
-
-  if (Headers && !dom::Headers_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (IOUtils && !dom::IOUtils_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (InspectorUtils && !dom::InspectorUtils_Binding::GetConstructorObject(cx))
-    return false;
-
-  if (MessageChannel &&
-      (!dom::MessageChannel_Binding::GetConstructorObject(cx) ||
-       !dom::MessagePort_Binding::GetConstructorObject(cx)))
-    return false;
-
-  if (Node && !dom::Node_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (NodeFilter && !dom::NodeFilter_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (PathUtils && !dom::PathUtils_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (Performance && !dom::Performance_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (PromiseDebugging &&
-      !dom::PromiseDebugging_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (Range && !dom::Range_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (Selection && !dom::Selection_Binding::GetConstructorObject(cx)) {
-    return false;
-  }
-
-  if (TextDecoder && !dom::TextDecoder_Binding::GetConstructorObject(cx))
-    return false;
-
-  if (TextEncoder && !dom::TextEncoder_Binding::GetConstructorObject(cx))
-    return false;
-
-  if (URL && !dom::URL_Binding::GetConstructorObject(cx)) return false;
-
-  if (URLSearchParams &&
-      !dom::URLSearchParams_Binding::GetConstructorObject(cx))
-    return false;
-
-  if (XMLHttpRequest && !dom::XMLHttpRequest_Binding::GetConstructorObject(cx))
-    return false;
-
-  if (WebSocket && !dom::WebSocket_Binding::GetConstructorObject(cx))
-    return false;
-
-  if (Window && !dom::Window_Binding::GetConstructorObject(cx)) return false;
-
-  if (XMLSerializer && !dom::XMLSerializer_Binding::GetConstructorObject(cx))
-    return false;
-
-  if (ReadableStream && !dom::ReadableStream_Binding::GetConstructorObject(cx))
-    return false;
+#undef DEFINE_WEBIDL_INTERFACE_OR_NAMESPACE
 
   if (atob && !JS_DefineFunction(cx, obj, "atob", Atob, 1, 0)) return false;
 
   if (btoa && !JS_DefineFunction(cx, obj, "btoa", Btoa, 1, 0)) return false;
 
-  if (caches && !dom::cache::CacheStorage::DefineCaches(cx, obj)) {
+  if (caches && !dom::cache::CacheStorage::DefineCachesForSandbox(cx, obj)) {
     return false;
   }
 
@@ -1421,7 +1342,7 @@ nsresult xpc::CreateSandboxObject(JSContext* cx, MutableHandleValue vp,
           return NS_ERROR_INVALID_ARG;
         }
         const JSClass* unwrappedClass = JS::GetClass(unwrappedProto);
-        useSandboxProxy = IS_WN_CLASS(unwrappedClass) ||
+        useSandboxProxy = unwrappedClass->isWrappedNative() ||
                           mozilla::dom::IsDOMClass(unwrappedClass);
       }
 
@@ -1443,9 +1364,15 @@ nsresult xpc::CreateSandboxObject(JSContext* cx, MutableHandleValue vp,
     }
 
     bool allowComponents = principal->IsSystemPrincipal();
-    if (options.wantComponents && allowComponents &&
-        !ObjectScope(sandbox)->AttachComponentsObject(cx))
-      return NS_ERROR_XPC_UNEXPECTED;
+    if (options.wantComponents && allowComponents) {
+      if (!ObjectScope(sandbox)->AttachComponentsObject(cx)) {
+        return NS_ERROR_XPC_UNEXPECTED;
+      }
+
+      if (!ObjectScope(sandbox)->AttachJSServices(cx)) {
+        return NS_ERROR_XPC_UNEXPECTED;
+      }
+    }
 
     if (!XPCNativeWrapper::AttachNewConstructorObject(cx, sandbox)) {
       return NS_ERROR_XPC_UNEXPECTED;
@@ -2230,3 +2157,24 @@ ModuleLoaderBase* SandboxPrivate::GetModuleLoader(JSContext* aCx) {
 
   return moduleLoader;
 }
+
+mozilla::Result<mozilla::ipc::PrincipalInfo, nsresult>
+SandboxPrivate::GetStorageKey() {
+  MOZ_ASSERT(NS_IsMainThread());
+
+  mozilla::ipc::PrincipalInfo principalInfo;
+  nsresult rv = PrincipalToPrincipalInfo(mPrincipal, &principalInfo);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return mozilla::Err(rv);
+  }
+
+  // Block expanded and null principals, let content and system through.
+  if (principalInfo.type() !=
+          mozilla::ipc::PrincipalInfo::TContentPrincipalInfo &&
+      principalInfo.type() !=
+          mozilla::ipc::PrincipalInfo::TSystemPrincipalInfo) {
+    return Err(NS_ERROR_DOM_SECURITY_ERR);
+  }
+
+  return std::move(principalInfo);
+}
