@@ -14,26 +14,28 @@
  * limitations under the License.
  */
 
-#include "ClearKeyDecryptionManager.h"
 #include "ClearKeySessionManager.h"
-#include "ClearKeyUtils.h"
-#include "ClearKeyStorage.h"
-#include "ClearKeyPersistence.h"
-// This include is required in order for content_decryption_module to work
-// on Unix systems.
-#include "stddef.h"
-#include "content_decryption_module.h"
-#include "psshparser/PsshParser.h"
 
 #include <assert.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <string.h>
 
-using namespace std;
+#include "content_decryption_module.h"
+
+#include "ClearKeyDecryptionManager.h"
+#include "ClearKeyPersistence.h"
+#include "ClearKeyStorage.h"
+#include "ClearKeyUtils.h"
+#include "psshparser/PsshParser.h"
+
 using namespace cdm;
 
-ClearKeySessionManager::ClearKeySessionManager(Host_9* aHost)
+using std::function;
+using std::string;
+using std::vector;
+
+ClearKeySessionManager::ClearKeySessionManager(Host_11* aHost)
     : mDecryptionManager(ClearKeyDecryptionManager::Get()) {
   CK_LOGD("ClearKeySessionManager ctor %p", this);
   AddRef();
@@ -58,10 +60,15 @@ void ClearKeySessionManager::Init(bool aDistinctiveIdentifierAllowed,
 
       func();
     }
+    if (self->mHost) {
+      // The session manager should be the last thing the ClearKey CDM is
+      // waiting on to be initialized.
+      self->mHost->OnInitialized(true);
+    }
   };
 
   mPersistence->EnsureInitialized(aPersistentStateAllowed,
-                                  move(onPersistentStateLoaded));
+                                  std::move(onPersistentStateLoaded));
 }
 
 void ClearKeySessionManager::CreateSession(uint32_t aPromiseId,
@@ -82,7 +89,7 @@ void ClearKeySessionManager::CreateSession(uint32_t aPromiseId,
   };
 
   // If we haven't loaded, don't do this yet
-  if (MaybeDeferTillInitialized(move(deferrer))) {
+  if (MaybeDeferTillInitialized(std::move(deferrer))) {
     CK_LOGD("Deferring CreateSession");
     return;
   }
@@ -125,6 +132,7 @@ void ClearKeySessionManager::CreateSession(uint32_t aPromiseId,
   }
 
   mSessions[sessionId] = session;
+  mLastSessionId = sessionId;
 
   const vector<KeyId>& sessionKeys = session->GetKeyIds();
   vector<KeyId> neededKeys;
@@ -171,7 +179,7 @@ void ClearKeySessionManager::LoadSession(uint32_t aPromiseId,
     self->LoadSession(aPromiseId, sessionId.data(), sessionId.size());
   };
 
-  if (MaybeDeferTillInitialized(move(deferrer))) {
+  if (MaybeDeferTillInitialized(std::move(deferrer))) {
     CK_LOGD("Deferring LoadSession");
     return;
   }
@@ -205,7 +213,7 @@ void ClearKeySessionManager::LoadSession(uint32_t aPromiseId,
     self->mHost->OnResolveNewSessionPromise(aPromiseId, nullptr, 0);
   };
 
-  ReadData(mHost, sessionId, move(success), move(failure));
+  ReadData(mHost, sessionId, std::move(success), std::move(failure));
 }
 
 void ClearKeySessionManager::PersistentSessionDataLoaded(
@@ -230,6 +238,7 @@ void ClearKeySessionManager::PersistentSessionDataLoaded(
       new ClearKeySession(aSessionId, SessionType::kPersistentLicense);
 
   mSessions[aSessionId] = session;
+  mLastSessionId = aSessionId;
 
   uint32_t numKeys = aKeyDataSize / (2 * CENC_KEY_LEN);
 
@@ -288,7 +297,7 @@ void ClearKeySessionManager::UpdateSession(uint32_t aPromiseId,
   };
 
   // If we haven't fully loaded, defer calling this method
-  if (MaybeDeferTillInitialized(move(deferrer))) {
+  if (MaybeDeferTillInitialized(std::move(deferrer))) {
     CK_LOGD("Deferring LoadSession");
     return;
   }
@@ -380,7 +389,7 @@ void ClearKeySessionManager::UpdateSession(uint32_t aPromiseId,
                                  message, strlen(message));
   };
 
-  WriteData(mHost, sessionId, keydata, move(resolve), move(reject));
+  WriteData(mHost, sessionId, keydata, std::move(resolve), std::move(reject));
 }
 
 void ClearKeySessionManager::Serialize(const ClearKeySession* aSession,
@@ -414,7 +423,7 @@ void ClearKeySessionManager::CloseSession(uint32_t aPromiseId,
   };
 
   // If we haven't loaded, call this method later.
-  if (MaybeDeferTillInitialized(move(deferrer))) {
+  if (MaybeDeferTillInitialized(std::move(deferrer))) {
     CK_LOGD("Deferring CloseSession");
     return;
   }
@@ -465,7 +474,7 @@ void ClearKeySessionManager::RemoveSession(uint32_t aPromiseId,
   };
 
   // If we haven't fully loaded, defer calling this method.
-  if (MaybeDeferTillInitialized(move(deferrer))) {
+  if (MaybeDeferTillInitialized(std::move(deferrer))) {
     CK_LOGD("Deferring RemoveSession");
     return;
   }
@@ -517,7 +526,8 @@ void ClearKeySessionManager::RemoveSession(uint32_t aPromiseId,
                                  message, strlen(message));
   };
 
-  WriteData(mHost, sessionId, emptyKeydata, move(resolve), move(reject));
+  WriteData(mHost, sessionId, emptyKeydata, std::move(resolve),
+            std::move(reject));
 }
 
 void ClearKeySessionManager::SetServerCertificate(uint32_t aPromiseId,
@@ -529,7 +539,7 @@ void ClearKeySessionManager::SetServerCertificate(uint32_t aPromiseId,
                          nullptr /* message */, 0 /* messageLen */);
 }
 
-Status ClearKeySessionManager::Decrypt(const InputBuffer_1& aBuffer,
+Status ClearKeySessionManager::Decrypt(const InputBuffer_2& aBuffer,
                                        DecryptedBlock* aDecryptedBlock) {
   CK_LOGD("ClearKeySessionManager::Decrypt");
 
@@ -563,6 +573,7 @@ void ClearKeySessionManager::DecryptingComplete() {
     delete it->second;
   }
   mSessions.clear();
+  mLastSessionId = std::nullopt;
 
   mDecryptionManager = nullptr;
   mHost = nullptr;
@@ -576,6 +587,127 @@ bool ClearKeySessionManager::MaybeDeferTillInitialized(
     return false;
   }
 
-  mDeferredInitialize.emplace(move(aMaybeDefer));
+  mDeferredInitialize.emplace(std::move(aMaybeDefer));
   return true;
 }
+
+void ClearKeySessionManager::OnQueryOutputProtectionStatus(
+    QueryResult aResult, uint32_t aLinkMask, uint32_t aOutputProtectionMask) {
+  MOZ_ASSERT(mHasOutstandingOutputProtectionQuery,
+             "Should only be called if a query is outstanding");
+  CK_LOGD("ClearKeySessionManager::OnQueryOutputProtectionStatus");
+  mHasOutstandingOutputProtectionQuery = false;
+
+  if (aResult == QueryResult::kQueryFailed) {
+    // Indicate the query failed. This can happen if we're in shutdown.
+    NotifyOutputProtectionStatus(KeyStatus::kInternalError);
+    return;
+  }
+
+  if (aLinkMask & OutputLinkTypes::kLinkTypeNetwork) {
+    NotifyOutputProtectionStatus(KeyStatus::kOutputRestricted);
+    return;
+  }
+
+  NotifyOutputProtectionStatus(KeyStatus::kUsable);
+}
+
+void ClearKeySessionManager::QueryOutputProtectionStatusIfNeeded() {
+  MOZ_ASSERT(
+      mHost,
+      "Should not query protection status if we're shutdown (mHost == null)!");
+  CK_LOGD(
+      "ClearKeySessionManager::UpdateOutputProtectionStatusAndQueryIfNeeded");
+  if (mLastOutputProtectionQueryTime.IsNull()) {
+    // We haven't perfomed a check yet, get a query going.
+    MOZ_ASSERT(
+        !mHasOutstandingOutputProtectionQuery,
+        "Shouldn't have an outstanding query if we haven't recorded a time");
+    QueryOutputProtectionStatusFromHost();
+    return;
+  }
+
+  MOZ_ASSERT(!mLastOutputProtectionQueryTime.IsNull(),
+             "Should have already handled the case where we don't yet have a "
+             "previous check time");
+  const mozilla::TimeStamp now = mozilla::TimeStamp::NowLoRes();
+  const mozilla::TimeDuration timeSinceQuery =
+      now - mLastOutputProtectionQueryTime;
+
+  // The time between output protection checks to the host. I.e. if this amount
+  // of time has passed since the last check with the host, another should be
+  // performed (provided the first check has been handled).
+  static const mozilla::TimeDuration kOutputProtectionQueryInterval =
+      mozilla::TimeDuration::FromSeconds(0.2);
+  // The number of kOutputProtectionQueryInterval intervals we can miss before
+  // we decide a check has failed. I.e. if this value is 2, if we have not
+  // received a reply to a check after kOutputProtectionQueryInterval * 2
+  // time, we consider the check failed.
+  constexpr uint32_t kMissedIntervalsBeforeFailure = 2;
+  // The length of time after which we will restrict output until we get a
+  // query response.
+  static const mozilla::TimeDuration kTimeToWaitBeforeFailure =
+      kOutputProtectionQueryInterval * kMissedIntervalsBeforeFailure;
+
+  if ((timeSinceQuery > kOutputProtectionQueryInterval) &&
+      !mHasOutstandingOutputProtectionQuery) {
+    // We don't have an outstanding query and enough time has passed we should
+    // query again.
+    QueryOutputProtectionStatusFromHost();
+    return;
+  }
+
+  if ((timeSinceQuery > kTimeToWaitBeforeFailure) &&
+      mHasOutstandingOutputProtectionQuery) {
+    // A reponse was not received fast enough, notify.
+    NotifyOutputProtectionStatus(KeyStatus::kInternalError);
+  }
+}
+
+void ClearKeySessionManager::QueryOutputProtectionStatusFromHost() {
+  MOZ_ASSERT(
+      mHost,
+      "Should not query protection status if we're shutdown (mHost == null)!");
+  CK_LOGD("ClearKeySessionManager::QueryOutputProtectionStatusFromHost");
+  if (mHost) {
+    mLastOutputProtectionQueryTime = mozilla::TimeStamp::NowLoRes();
+    mHost->QueryOutputProtectionStatus();
+    mHasOutstandingOutputProtectionQuery = true;
+  }
+}
+
+void ClearKeySessionManager::NotifyOutputProtectionStatus(KeyStatus aStatus) {
+  MOZ_ASSERT(aStatus == KeyStatus::kUsable ||
+                 aStatus == KeyStatus::kOutputRestricted ||
+                 aStatus == KeyStatus::kInternalError,
+             "aStatus should have an expected value");
+  CK_LOGD("ClearKeySessionManager::NotifyOutputProtectionStatus");
+  if (!mLastSessionId.has_value()) {
+    // If we don't have a session id, either because we're too early, or are
+    // shutting down, don't notify.
+    return;
+  }
+
+  string& lastSessionId = mLastSessionId.value();
+
+  // Use 'output-protection' as the key ID. This helps tests disambiguate key
+  // status updates related to this.
+  const uint8_t kKeyId[] = {'o', 'u', 't', 'p', 'u', 't', '-', 'p', 'r',
+                            'o', 't', 'e', 'c', 't', 'i', 'o', 'n'};
+  KeyInformation keyInfo = {};
+  keyInfo.key_id = kKeyId;
+  keyInfo.key_id_size = std::size(kKeyId);
+  keyInfo.status = aStatus;
+
+  vector<KeyInformation> keyInfos;
+  keyInfos.push_back(keyInfo);
+
+  // At time of writing, Gecko's higher level handling doesn't use this arg.
+  // However, we set it to false to mimic Chromium's similar case. Since
+  // Clearkey is used to test the Chromium CDM path, it doesn't hurt to try
+  // and mimic their behaviour.
+  bool hasAdditionalUseableKey = false;
+  mHost->OnSessionKeysChange(lastSessionId.c_str(), lastSessionId.size(),
+                             hasAdditionalUseableKey, keyInfos.data(),
+                             keyInfos.size());
+}
